<?php
/**
 * ```
 *  _ _ _     __  __ _ _ _            ____  _   _ ____
 * | (_) |__ |  \/  (_) | |_ ___ _ __|  _ \| | | |  _ \
 * | | | '_ \| |\/| | | | __/ _ \ '__| |_) | |_| | |_) |
 * | | | |_) | |  | | | | ||  __/ |  |  __/|  _  |  __/
 * |_|_|_.__/|_|  |_|_|_|_| \___|_|  |_|   |_| |_|_|
 * ```
 *
 */

namespace libMilterPHP;

/**
 * ReplyHelper trait
 *
 * These are helper/wrapper functions around the reply() function.
 *
 * @copyright Noumenia (C) 2019 - All rights reserved - Software Development - www.noumenia.gr
 * @license GNU GPL v3.0
 * @package libMilterPHP
 * @subpackage replyhelper
 */
trait ReplyHelper {

	/**
	 * Add recipient
	 * @param string $email
	 * @return bool
	 */
	public function replyAddRecipient(string $email): bool
	{

		$reply = array(

			// Set the command (char)
			'command'	=> SMFIR_ADDRCPT,

			// Set the data (chars of uint32 size)
			'data'		=> array('rcpt' => $email)

		);

		return $this->reply($reply);

	}

	/**
	 * Delete recipient
	 * @param string $email
	 * @return bool
	 */
	public function replyDeleteRecipient(string $email): bool
	{

		$reply = array(

			// Set the command (char)
			'command'	=> SMFIR_DELRCPT,

			// Set the data (chars of uint32 size)
			'data'		=> array('rcpt' => $email)

		);

		return $this->reply($reply);

	}

	/**
	 * Accept message completely
	 * @return bool
	 */
	public function replyAccept(): bool
	{

		$reply = array(

			// Set the command (char)
			'command'	=> SMFIR_ACCEPT,

			// Set the data (chars of uint32 size)
			'data'		=> array()

		);

		return $this->reply($reply);

	}

	/**
	 * Replace body chunk
	 * @param string $text
	 * @return bool
	 */
	public function replyReplaceBodyChunk(string $text): bool
	{

		$reply = array(

			// Set the command (char)
			'command'	=> SMFIR_REPLBODY,

			// Set the data (chars of uint32 size)
			'data'		=> array('buf' => $text)

		);

		return $this->reply($reply);

	}

	/**
	 * Continue
	 * @return bool
	 */
	public function replyContinue(): bool
	{

		$reply = array(

			// Set the command (char)
			'command'	=> SMFIR_CONTINUE,

			// Set the data (chars of uint32 size)
			'data'		=> array()

		);

		return $this->reply($reply);

	}

	/**
	 * Discard
	 * @return bool
	 */
	public function replyDiscard(): bool
	{

		$reply = array(

			// Set the command (char)
			'command'	=> SMFIR_DISCARD,

			// Set the data (chars of uint32 size)
			'data'		=> array()

		);

		return $this->reply($reply);

	}

	/**
	 * Add header
	 * @param string $key
	 * @param string $value
	 * @return bool
	 */
	public function replyAddHeader(string $key, string $value): bool
	{

		$reply = array(

			// Set the command (char)
			'command'	=> SMFIR_ADDHEADER,

			// Set the data (chars of uint32 size)
			'data'		=> array('name' => $key, 'value' => $value)

		);

		return $this->reply($reply);

	}

	/**
	 * Change header
	 * @param string $key
	 * @param string $value
	 * @param int $index Index of the occurrence of this header
	 * @return bool
	 */
	public function replyChangeHeader(string $key, string $value, int $index = 1): bool
	{

		$reply = array(

			// Set the command (char)
			'command'	=> SMFIR_CHGHEADER,

			// Set the data (chars of uint32 size)
			'data'		=> array('index' => $index, 'name' => $key, 'value' => $value)

		);

		return $this->reply($reply);

	}

	/**
	 * Delete header
	 * @param string $key
	 * @param int $index Index of the occurrence of this header
	 * @return bool
	 */
	public function replyDeleteHeader(string $key, int $index = 1): bool
	{

		$reply = array(

			// Set the command (char)
			'command'	=> SMFIR_CHGHEADER,

			// Set the data (chars of uint32 size)
			'data'		=> array('index' => $index, 'name' => $key, 'value' => "")

		);

		return $this->reply($reply);

	}

	/**
	 * Progress
	 * @return bool
	 */
	public function replyProgress(): bool
	{

		$reply = array(

			// Set the command (char)
			'command'	=> SMFIR_PROGRESS,

			// Set the data (chars of uint32 size)
			'data'		=> array()

		);

		return $this->reply($reply);

	}

	/**
	 * Quarantine
	 * @param string $reason
	 * @return bool
	 */
	public function replyQuarantine(string $reason): bool
	{

		$reply = array(

			// Set the command (char)
			'command'	=> SMFIR_QUARANTINE,

			// Set the data (chars of uint32 size)
			'data'		=> array('reason' => $reason)

		);

		return $this->reply($reply);

	}

	/**
	 * Reject command/recipient with a 550 5.7.1
	 * @param string $text
	 * @return bool
	 */
	public function replyReject(string $text = "Blocked by libMilterPHP"): bool
	{

		$reply = array(

			// Set the command (char)
			'command'	=> SMFIR_REPLYCODE,

			// Set the data (chars of uint32 size)
			'data'		=> array(
				'smtpcode'	=> "550",
				'space'		=> " ",
				'text'		=> "5.7.1 " . $text
			)

		);

		return $this->reply($reply);

	}

	/**
	 * Reject command/recipient with a 4xx (Tempfail)
	 * @return bool
	 */
	public function replyTempFail(): bool
	{

		$reply = array(

			// Set the command (char)
			'command'	=> SMFIR_TEMPFAIL,

			// Set the data (chars of uint32 size)
			'data'		=> array()

		);

		return $this->reply($reply);

	}

	/**
	 * Reply code
	 * @param string $smtpCode
	 * @param string $text
	 * @return bool
	 */
	public function replyReplyCode(string $smtpCode, string $text): bool
	{

		$reply = array(

			// Set the command (char)
			'command'	=> SMFIR_REPLYCODE,

			// Set the data (chars of uint32 size)
			'data'		=> array(
				'smtpcode'	=> $smtpCode,
				'space'		=> " ",
				'text'		=> $text
			)

		);

		return $this->reply($reply);

	}

}

